/*
	Signals test program.
	
	This program spawns a subprocess which will use abort() to end. It is
	expected that this parent will then be able to capture that signal as part
	of why it is killed.
*/

#include <cstdlib>
#include <iostream>

#include <signal.h>
#include <spawn.h>
#include <sys/wait.h>


int
main(int argc, char** argv)
{
	// Check if we are the child process; this is the case we have arguments.
	if (argc > 1)
		std::abort();

	// Main process: set up a child process and capture on which signal it terminated
	pid_t childPid;
	char* childArgv[] = {argv[0], "abort", nullptr};
	if (auto status = posix_spawn(&childPid, argv[0], nullptr, nullptr, childArgv, nullptr);
			status != 0)
	{
		std::cerr << "Error spawning process: " << strerror(status) << std::endl;
		return status;
	}

	int childStatus;
	auto returnPid = waitpid(childPid, &childStatus, 0);
	if (returnPid != childPid) {
		std::cerr << "waitpid() did not return the requested pid" << std::endl;
		return -1;
	}
	if (!WIFSIGNALED(childStatus) != 0) {
		std::cerr << "waidpid(): the child process was expected to stop due to an uncaught signal"
			<< std::endl;
		return -1;
	}
	if (auto signal = WTERMSIG(childStatus); signal != SIGABRT) {
		std::cerr << "the child process ended with signal " << signal << " instead of SIGABRT"
			<< std::endl;
		return -1;
	}
	
	std::cout << "Test succeeded: child signal SIGABRT was transmitted correctly" << std::endl;
	return 0;
}
